<?php

declare(strict_types=1);

namespace Drupal\data_pipelines_sftp\Source\Resource;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\data_pipelines\Entity\DatasetInterface;
use Drupal\data_pipelines\Source\Resource\SourceResourceBase;
use Drupal\key\KeyRepositoryInterface;
use phpseclib3\Net\SFTP as SFTPClient;
use phpseclib3\Net\SFTP\Stream;

/**
 * Provides SFTP a source resource.
 */
class Sftp extends SourceResourceBase {

  /**
   * The dataset.
   *
   * @var Drupal\data_pipelines\Entity\DatasetInterface
   */
  protected DatasetInterface $dataset;

  /**
   * The field name.
   *
   * @var string
   */
  protected string $fieldName;

  public function __construct(protected KeyRepositoryInterface $keyRepository) {}

  /**
   * {@inheritdoc}
   */
  public function getResource(DatasetInterface $dataset, string $field_name): mixed {
    $this->dataset = $dataset;
    $this->fieldName = $field_name;
    try {
      $client = $this->getClient();
      return $this->requireLocalCopy() ? $this->createLocalCopy($client) : $this->createMemoryCopy($client);
    }
    catch (\Exception $e) {
      $dataset->addLogMessage(sprintf("There was an error getting the file from the file server - %s", $e->getMessage()));
    }
    if ($this->requireLocalCopy()) {
      $dataset->addLogMessage(sprintf("The local copy of %s has been used.", $this->getPath()));
      return $this->getLocalCopy();
    }
    return NULL;
  }

  /**
   * A method to get the file path.
   *
   * @return string
   *   The file path.
   */
  protected function getPath(): string {
    return self::getFieldValue($this->dataset, $this->fieldName);
  }

  /**
   * A method to determine whether a local copy is required.
   *
   * @return bool
   *   The result.
   */
  protected function requireLocalCopy(): bool {
    return (bool) self::getFieldValue($this->dataset, $this->fieldName, 'local_copy');
  }

  /**
   * A method to get the host.
   *
   * @return string
   *   The host.
   */
  protected function getHost(): string {
    return self::getFieldValue($this->dataset, $this->fieldName, 'host');
  }

  /**
   * A method to get the port.
   *
   * @return int
   *   The port.
   */
  protected function getPort(): int {
    return (int) self::getFieldValue($this->dataset, $this->fieldName, 'port');
  }

  /**
   * A method to get the credentials.
   *
   * @return string
   *   The credentials.
   */
  protected function getCredentials(): string {
    return self::getFieldValue($this->dataset, $this->fieldName, 'credentials');
  }

  /**
   * A method to get the authenticated SFTP client.
   *
   * @return \phpseclib3\Net\SFTP
   *   The authenticated SFTP client.
   *
   * @throws \Exception
   */
  protected function getClient(): SFTPClient {
    static $client;
    if ($client) {
      return $client;
    }
    Stream::register();
    $client = new SFTPClient($this->getHost(), $this->getPort());
    ['username' => $username, 'password' => $password] = Json::decode($this->keyRepository->getKey($this->getCredentials())->getKeyValue());
    if (!$client->login($username, $password)) {
      throw new \Exception('Authentication failed.');
    }
    return $client;
  }

  /**
   * A method to get a local copy of the file if opted into.
   *
   * This method is used as a fallback - if the file cannot be fetched and
   * a local copy is available.
   *
   * @return mixed
   *   The resource.
   */
  protected function getLocalCopy(): mixed {
    $filename = pathinfo($this->getPath(), PATHINFO_BASENAME);
    $local_path = 'private://' . $filename;
    if (is_readable($local_path)) {
      if ($local_stream = fopen($local_path, 'r')) {
        rewind($local_stream);
        return $local_stream;
      }
    }
    return NULL;
  }

  /**
   * A method to create a local copy of the file if opted into.
   *
   * @param \phpseclib3\Net\SFTP $client
   *   The authenticated SFTP client.
   *
   * @return string
   *   The resource.
   *
   * @throws \Exception
   */
  protected function createLocalCopy(SFTPClient $client): mixed {
    $filename = pathinfo($this->getPath(), PATHINFO_BASENAME);
    $local_path = 'private://' . $filename;
    if ($local_stream = fopen($local_path, 'a+')) {
      if (!fwrite($local_stream, $client->get($this->getPath()))) {
        throw new \Exception('Could not create local copy.');
      }
      rewind($local_stream);
      return $local_stream;
    }
    return NULL;
  }

  /**
   * A method to create a memory copy of the file.
   *
   * @param \phpseclib3\Net\SFTP $client
   *   The authenticated SFTP client.
   *
   * @return string
   *   The resource.
   *
   * @throws \Exception
   */
  protected function createMemoryCopy(SFTPClient $client): mixed {
    $memory_path = 'php://temp';
    if ($memory_stream = fopen($memory_path, 'w+')) {
      if (!fwrite($memory_stream, $client->get($this->getPath()))) {
        throw new \Exception('Could not create memory copy.');
      }
      rewind($memory_stream);
      return $memory_stream;
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getResourceBaseFieldDefinition(array $source_plugin_definition): BaseFieldDefinition {
    return BaseFieldDefinition::create('sftp')
      ->setLabel(new TranslatableMarkup('SFTP details'))
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'sftp',
      ]);
  }

}
