<?php

declare(strict_types=1);

namespace Drupa\Tests\data_pipelines_sftp\Unit\Source\Resource;

use Drupal\data_pipelines\Entity\DatasetInterface;
use Drupal\data_pipelines_sftp\Source\Resource\Sftp;
use phpseclib3\Net\SFTP as SFTPClient;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

/**
 * @covers \Drupal\data_pipelines_sftp\Source\Resource\Sftp
 */
class SftpTest extends TestCase {

  /**
   * A method to get the mocked Sftp class.
   *
   * @return PHPUnit\Framework\MockObject\MockObject|Drupal\data_pipelines_sftp\Source\Resource\Sftp
   *   The mocked Sftp class.
   */
  protected function getBaseMock(): MockObject | Sftp {
    $mock = $this->getMockBuilder(Sftp::class)
      ->onlyMethods([
        'getPath',
        'requireLocalCopy',
        'getHost',
        'getPort',
        'getCredentials',
        'getClient',
      ])
      ->disableOriginalConstructor()
      ->disableOriginalClone()
      ->disableArgumentCloning()
      ->disallowMockingUnknownTypes()
      ->getMock();
    $mock->expects($this->any())
      ->method('getPath')
      ->willReturn('foo/bar.json');
    $mock->expects($this->any())
      ->method('requireLocalCopy')
      ->willReturn(FALSE);
    $mock->expects($this->any())
      ->method('getHost')
      ->willReturn('127.0.0.1');
    $mock->expects($this->any())
      ->method('getPort')
      ->willReturn(22);
    $mock->expects($this->any())
      ->method('getCredentials')
      ->willReturn('123456789');
    return $mock;
  }

  /**
   * A method to get dummy data.
   *
   * @return string
   *   The data.
   */
  protected function getData(): string {
    return '{"foo": "bar"}';
  }

  /**
   * A method to test writing to memory and getting a resource.
   *
   * @covers \Drupal\data_pipelines_sftp\Source\Resource\Sftp::getResource
   */
  public function testGetResourceMemorySuccess(): void {
    $mock = $this->getBaseMock();
    $dataset = $this->createMock(DatasetInterface::class);

    // Memory write success.
    $client = $this->createMock(SFTPClient::class);
    $client->expects($this->any())
      ->method('get')
      ->withAnyParameters()
      ->willReturn($this->getData());
    $mock->expects($this->any())
      ->method('getClient')
      ->willReturn($client);
    $resource = $mock->getResource($dataset, 'foo_bar');
    $this->assertTrue(is_resource($resource));
    $this->assertEquals($this->getData(), fread($resource, 1024));
  }

  /**
   * A method to test writing to memory and not getting a resource.
   *
   * @covers \Drupal\data_pipelines_sftp\Source\Resource\Sftp::getResource
   */
  public function testGetResourceMemoryFail(): void {
    $mock = $this->getBaseMock();
    $dataset = $this->createMock(DatasetInterface::class);
    $client = $this->createMock(SFTPClient::class);
    $client->expects($this->any())
      ->method('get')
      ->withAnyParameters()
      ->willThrowException(new \Exception('Memory write failed.'));
    $mock->expects($this->any())
      ->method('getClient')
      ->willReturn($client);
    $mock->expects($this->any())
      ->method('requireLocalCopy')
      ->willReturn(FALSE);
    $resource = $mock->getResource($dataset, 'foo_bar');
    $this->assertFalse(is_resource($resource));
  }

  /**
   * A method to test writing to a local file and getting a resource.
   *
   * @covers \Drupal\data_pipelines_sftp\Source\Resource\Sftp::getResource
   */
  public function testGetResourceLocalFileSuccess(): void {
    $mock = $this->getBaseMock();
    $dataset = $this->createMock(DatasetInterface::class);
    $client = $this->createMock(SFTPClient::class);
    $client->expects($this->any())
      ->method('get')
      ->withAnyParameters()
      ->willReturn($this->getData());
    $mock->expects($this->any())
      ->method('getClient')
      ->willReturn($client);
    $mock->expects($this->any())
      ->method('requireLocalCopy')
      ->willReturn(TRUE);
    $resource = $mock->getResource($dataset, 'foo_bar');
    $this->assertTrue(is_resource($resource));
    $this->assertEquals($this->getData(), fread($resource, 1024));
  }

  /**
   * A method to test writing to a local file and not getting a resource.
   *
   * @covers \Drupal\data_pipelines_sftp\Source\Resource\Sftp::getResource
   */
  public function testGetResourceLocalFileFail(): void {
    $mock = $this->getBaseMock();
    $dataset = $this->createMock(DatasetInterface::class);
    $client = $this->createMock(SFTPClient::class);
    $client->expects($this->any())
      ->method('get')
      ->willThrowException(new \Exception('Local file write failed.'));
    $mock->expects($this->any())
      ->method('getClient')
      ->willReturn($client);
    $mock->expects($this->any())
      ->method('requireLocalCopy')
      ->willReturn(TRUE);
    $resource = $mock->getResource($dataset, 'foo_bar');
    $this->assertFalse(is_resource($resource));
  }

  /**
   * A method to test failed client authentication and not getting a resource.
   *
   * @covers \Drupal\data_pipelines_sftp\Source\Resource\Sftp::getResource
   */
  public function testGetResourceAuthenticationFail(): void {
    $mock = $this->getBaseMock();
    $dataset = $this->createMock(DatasetInterface::class);
    $mock->expects($this->any())
      ->method('getClient')
      ->willThrowException(new \Exception('Authentication failed.'));
    $resource = $mock->getResource($dataset, 'foo_bar');
    $this->assertFalse(is_resource($resource));
  }

}
